@@ -1,3 +1,17 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::Test;
 
 use strict;
@@ -16,7 +30,7 @@ use vars qw(@ISA @EXPORT $VERSION %SubTests @SkipReasons);
              have_min_apache_version have_apache_version have_perl 
              have_min_perl_version have_min_module_version
              have_threads under_construction have_apache_mpm);
-$VERSION = '1.06';
+$VERSION = '1.09';
 
 %SubTests = ();
 @SkipReasons = ();
@@ -32,7 +46,7 @@ sub config {
 }
 
 sub vars {
-    config()->{vars};
+    @_ ? @{ config()->{vars} }{ @_ } : config()->{vars};
 }
 
 sub sok (&;$) {
@@ -225,8 +239,7 @@ sub have_min_module_version {
     # have_module requires the perl module
     return 0 unless have_module($module);
 
-    my $has_version = $module->VERSION || 0;
-    return 1 if $has_version >= $version;
+    return 1 if eval { $module->VERSION($version) };
 
     push @SkipReasons, "$module version $version or higher is required";
     return 0;
@@ -264,7 +277,7 @@ sub have_min_apache_version {
     my $cfg = Apache::Test::config();
     (my $current) = $cfg->{server}->{version} =~ m:^Apache/(\d\.\d+\.\d+):;
 
-    if ($current lt $wanted) {
+    if (normalize_vstring($current) < normalize_vstring($wanted)) {
         push @SkipReasons,
           "apache version $wanted or higher is required," .
           " this is version $current";
@@ -280,7 +293,7 @@ sub have_apache_version {
     my $cfg = Apache::Test::config();
     (my $current) = $cfg->{server}->{version} =~ m:^Apache/(\d\.\d+\.\d+):;
 
-    if ($current ne $wanted) {
+    if (normalize_vstring($current) != normalize_vstring($wanted)) {
         push @SkipReasons,
           "apache version $wanted or higher is required," .
           " this is version $current";
@@ -372,6 +385,16 @@ sub under_construction {
     return 0;
 }
 
+# normalize Apache-sytle version strings (2.0.48, 0.9.4)
+# for easy numeric comparison.  note that 2.1 and 2.1.0
+# are considered equivalent.
+sub normalize_vstring {
+
+    my @digits = shift =~ m/(\d+)\.?(\d*)\.?(\d*)/;
+
+    return join '', map { sprintf("%03d", $_ || 0) } @digits;
+}
+
 package Apache::TestToString;
 
 sub TIEHANDLE {
@@ -682,6 +705,29 @@ reference, but be careful that the keys will be different.
 
 Also see plan().
 
+=item config
+
+  my $cfg = Apache::Test::config();
+  my $server_rev = $cfg->{server}->{rev};
+  ...
+
+C<config()> gives an access to the configuration object.
+
+=item vars
+
+  my $serverroot = Apache::Test::vars->{serverroot};
+  my $serverroot = Apache::Test::vars('serverroot');
+  my($top_dir, $t_dir) = Apache::Test::vars(qw(top_dir t_dir));
+
+C<vars()> gives an access to the configuration variables, otherwise
+accessible as:
+
+  $vars = Apache::Test::config()->{vars};
+
+If no arguments are passed, the reference to the variables hash is
+returned. If one or more arguments are passed the corresponding values
+are returned.
+
 =back
 
 =head1 Apache::TestToString Class
