@@ -1,3 +1,17 @@
+# Copyright 2000-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::Build;
 
 use 5.006;
@@ -1023,10 +1037,22 @@ sub httpd_version {
             $version = $1;
             last;
         }
+        elsif(/^\#define\s+AP_SERVER_MAJORVERSION_NUMBER\s+(\d+)/) {
+            # new 2.1 config
+            my $major = $1;
+            my $minor = (split /\s+/, scalar(<$fh>))[-1];
+            my $patch = (split /\s+/, scalar(<$fh>))[-1];
+            my $string = (split /\s+/, scalar(<$fh>))[-1];
+            $version = join '.', $major, $minor, "$patch$string";
+            $version =~ s/\"//g;
+            last;
+        }
     }
 
     close $fh;
 
+    debug "parsed version $version from ap_release.h";
+
     $self->httpd_version_cache($dir, $version);
 }
 
