@@ -1,3 +1,17 @@
+# Copyright 2002-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package ModPerl::TestRun;
 
 use strict;
@@ -5,21 +19,21 @@ use warnings FATAL => 'all';
 
 use base qw(Apache::TestRunPerl);
 
+use Apache::Build;
+
 # some mp2 tests require more than one server instance to be available
 # without which the server may hang, waiting for the single server
 # become available
 use constant MIN_MAXCLIENTS => 2;
 
-use Apache::Build;
-my $build = Apache::Build->build_config;
-
 sub new_test_config {
     my $self = shift;
 
-    # timeout in secs (threaded mpms are extremely slow to startup,
-    # due to a slow perl_clone operation)
-    $self->{conf_opts}->{startup_timeout} =
-        $build->mpm_is_threaded() ? 180 : 120;
+    # default timeout in secs (threaded mpms are extremely slow to
+    # startup, due to a slow perl_clone operation)
+    $self->{conf_opts}->{startup_timeout} ||=
+        $ENV{APACHE_TEST_STARTUP_TIMEOUT} ||
+        Apache::Build->build_config->mpm_is_threaded() ? 300 : 120;
 
     $self->{conf_opts}->{maxclients} ||= MIN_MAXCLIENTS;
 
