@@ -1,5 +1,56 @@
+/* Copyright 2001-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 #include "mod_perl.h"
 
+#ifdef USE_ITHREADS
+
+/*
+ * perl context overriding and restoration is required when
+ * PerlOptions +Parent/+Clone is used in vhosts, and perl is used to
+ * at the server startup. So that <Perl> sections, PerlLoadModule,
+ * PerlModule and PerlRequire are all run using the right perl context
+ * and restore to the original context when they are done.
+ *
+ * As of perl-5.8.3 it's unfortunate that it uses PERL_GET_CONTEXT and
+ * doesn't rely on the passed pTHX internally. When and if perl is
+ * fixed to always use pTHX if available, this context switching mess
+ * can be removed.
+ */
+
+#define MP_PERL_DECLARE_CONTEXT \
+    PerlInterpreter *orig_perl; \
+    pTHX;
+
+/* XXX: .htaccess support cannot use this perl with threaded MPMs */
+#define MP_PERL_OVERRIDE_CONTEXT    \
+    orig_perl = PERL_GET_CONTEXT;   \
+    aTHX = scfg->mip->parent->perl; \
+    PERL_SET_CONTEXT(aTHX);
+
+#define MP_PERL_RESTORE_CONTEXT     \
+    PERL_SET_CONTEXT(orig_perl);
+
+#else
+
+#define MP_PERL_DECLARE_CONTEXT
+#define MP_PERL_OVERRIDE_CONTEXT
+#define MP_PERL_RESTORE_CONTEXT
+
+#endif
+
 static char *modperl_cmd_unclosed_directive(cmd_parms *parms)
 {
     return apr_pstrcat(parms->pool, parms->cmd->name,
@@ -30,6 +81,41 @@ char *modperl_cmd_push_handlers(MpAV **handlers, const char *name,
     return NULL;
 }
 
+char *modperl_cmd_push_filter_handlers(MpAV **handlers,
+                                       const char *name,
+                                       apr_pool_t *p)
+{
+    modperl_handler_t *h = modperl_handler_new(p, name);
+
+    /* filter modules need to be autoloaded, because their attributes
+     * need to be known long before the callback is issued
+     */
+    if (*name == '-') {
+        MP_TRACE_h(MP_FUNC,
+                   "[%s] warning: filter handler %s will be not autoloaded. "
+                   "Unless the module defining this handler is explicitly "
+                   "preloaded, filter attributes will be ignored.\n",
+                   modperl_pid_tid(p), h->name);
+    }
+    else {
+        MpHandlerAUTOLOAD_On(h);
+        MP_TRACE_h(MP_FUNC,
+                   "[%s] filter handler %s will be autoloaded (to make "
+                   "the filter attributes available)\n",
+                   modperl_pid_tid(p), h->name);
+    }
+    
+    if (!*handlers) {
+        *handlers = modperl_handler_array_new(p);
+        MP_TRACE_d(MP_FUNC, "created handler stack\n");
+    }
+
+    modperl_handler_array_push(*handlers, h);
+    MP_TRACE_d(MP_FUNC, "pushed httpd filter handler: %s\n", h->name);
+
+    return NULL;
+}
+
 char *modperl_cmd_push_httpd_filter_handlers(MpAV **handlers,
                                              const char *name,
                                              apr_pool_t *p)
@@ -105,6 +191,7 @@ MP_CMD_SRV_DECLARE(switches)
 MP_CMD_SRV_DECLARE(modules)
 {
     MP_dSCFG(parms->server);
+    MP_PERL_DECLARE_CONTEXT;
 
     if (modperl_is_running() &&
         modperl_init_vhost(parms->server, parms->pool, NULL) != OK)
@@ -113,27 +200,29 @@ MP_CMD_SRV_DECLARE(modules)
     }
 
     if (modperl_is_running()) {
-#ifdef USE_ITHREADS
-        /* XXX: .htaccess support cannot use this perl with threaded MPMs */
-        dTHXa(scfg->mip->parent->perl);
-#endif
-        MP_TRACE_d(MP_FUNC, "load PerlModule %s\n", arg);
+        char *error = NULL;
 
+        MP_TRACE_d(MP_FUNC, "load PerlModule %s\n", arg);
+        
+        MP_PERL_OVERRIDE_CONTEXT;
         if (!modperl_require_module(aTHX_ arg, FALSE)) {
-            return SvPVX(ERRSV);
+            error = SvPVX(ERRSV);
         }
+        MP_PERL_RESTORE_CONTEXT;
+
+        return error;
     }
     else {
         MP_TRACE_d(MP_FUNC, "push PerlModule %s\n", arg);
         *(const char **)apr_array_push(scfg->PerlModule) = arg;
+        return NULL;
     }
-
-    return NULL;
 }
 
 MP_CMD_SRV_DECLARE(requires)
 {
     MP_dSCFG(parms->server);
+    MP_PERL_DECLARE_CONTEXT;
 
     if (modperl_is_running() &&
         modperl_init_vhost(parms->server, parms->pool, NULL) != OK)
@@ -142,23 +231,23 @@ MP_CMD_SRV_DECLARE(requires)
     }
 
     if (modperl_is_running()) {
-#ifdef USE_ITHREADS
-        /* XXX: .htaccess support cannot use this perl with threaded MPMs */
-        dTHXa(scfg->mip->parent->perl);
-#endif
+        char *error = NULL;
 
         MP_TRACE_d(MP_FUNC, "load PerlRequire %s\n", arg);
 
+        MP_PERL_OVERRIDE_CONTEXT;
         if (!modperl_require_file(aTHX_ arg, FALSE)) {
-            return SvPVX(ERRSV);
+            error = SvPVX(ERRSV);
         }
+        MP_PERL_RESTORE_CONTEXT;
+
+        return error;
     }
     else {
         MP_TRACE_d(MP_FUNC, "push PerlRequire %s\n", arg);
         *(const char **)apr_array_push(scfg->PerlRequire) = arg;
+        return NULL;
     }
-
-    return NULL;
 }
 
 static MP_CMD_SRV_DECLARE2(handle_vars)
@@ -167,17 +256,42 @@ static MP_CMD_SRV_DECLARE2(handle_vars)
     modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
     const char *name = parms->cmd->name;
 
+    /* PerlSetVar and PerlAddVar logic.  here's the deal...
+     *
+     * cfg->configvars holds the final PerlSetVar/PerlAddVar configuration
+     * for a given server or directory.  however, getting to that point
+     * is kind of tricky, due to the add-style nature of PerlAddVar.
+     *
+     * the solution is to use cfg->setvars to hold PerlSetVar entries
+     * and cfg->addvars to hold PerlAddVar entries, each serving as a
+     * placeholder for when we need to know what's what in the merge routines.
+     *
+     * however, for the initial pass, apr_table_setn and apr_table_addn
+     * will properly build the configvars table, which will be visible to
+     * startup scripts trying to access per-server configurations.
+     *
+     * the end result is that we need to populate all three tables in order
+     * to keep things straight later on see merge_table_config_vars in
+     * modperl_config.c
+     */
     modperl_table_modify_t func =
         strEQ(name, "PerlSetVar") ? apr_table_setn : apr_table_addn;
 
-    func(dcfg->vars, arg1, arg2);
+    apr_table_t *table =
+        strEQ(name, "PerlSetVar") ? dcfg->setvars : dcfg->addvars;
+
+    func(table, arg1, arg2);
+    func(dcfg->configvars, arg1, arg2);
 
     MP_TRACE_d(MP_FUNC, "%s DIR: arg1 = %s, arg2 = %s\n",
                name, arg1, arg2);
 
     /* make available via Apache->server->dir_config */
     if (!parms->path) {
-        func(scfg->vars, arg1, arg2);
+        table = strEQ(name, "PerlSetVar") ? scfg->setvars : scfg->addvars;
+
+        func(table, arg1, arg2);
+        func(scfg->configvars, arg1, arg2);
 
         MP_TRACE_d(MP_FUNC, "%s SRV: arg1 = %s, arg2 = %s\n",
                    name, arg1, arg2);
@@ -332,7 +446,7 @@ MP_CMD_SRV_DECLARE(perl)
         /*XXX: Less than optimal */
         code = apr_pstrcat(p, code, line, "\n", NULL);
     }
-    
+
     /* Here, we have to replace our current config node for the next pass */
     if (!*current) {
         *current = apr_pcalloc(p, sizeof(**current));
@@ -358,21 +472,13 @@ MP_CMD_SRV_DECLARE(perldo)
 {
     apr_pool_t *p = parms->pool;
     server_rec *s = parms->server;
-    apr_table_t *options = NULL;
-    const char *handler_name = NULL;
+    apr_table_t *options;
     modperl_handler_t *handler = NULL;
-    const char *pkg_base = NULL;
-    const char *pkg_namespace = NULL;
     const char *pkg_name = NULL;
-    const char *line_header = NULL;
     ap_directive_t *directive = parms->directive;
-    int status = OK;
-    AV *args = Nullav;
-    SV *dollar_zero = Nullsv;
-    int dollar_zero_tainted;
 #ifdef USE_ITHREADS
     MP_dSCFG(s);
-    pTHX;
+    MP_PERL_DECLARE_CONTEXT;
 #endif
 
     if (!(arg && *arg)) {
@@ -386,13 +492,15 @@ MP_CMD_SRV_DECLARE(perldo)
         return "init mod_perl vhost failed";
     }
     
-#ifdef USE_ITHREADS
-    /* XXX: .htaccess support cannot use this perl with threaded MPMs */
-    aTHX = scfg->mip->parent->perl;
-#endif
+    MP_PERL_OVERRIDE_CONTEXT;
 
     /* data will be set by a <Perl> section */
-    if ((options = parms->directive->data)) {
+    if ((options = directive->data)) {
+        const char *pkg_namespace;
+        const char *pkg_base;
+        const char *handler_name;
+        const char *line_header;
+
         if (!(handler_name = apr_table_get(options, "handler"))) {
             handler_name = apr_pstrdup(p, MP_DEFAULT_PERLSECTION_HANDLER);
             apr_table_set(options, "handler", handler_name);
@@ -421,41 +529,36 @@ MP_CMD_SRV_DECLARE(perldo)
         arg = apr_pstrcat(p, "package ", pkg_name, ";", line_header,
                           arg, NULL);
     }
-
-    /* Set $0 to the current configuration file */
-    dollar_zero = get_sv("0", TRUE);
-    dollar_zero_tainted = SvTAINTED(dollar_zero);
-
-    if (dollar_zero_tainted) {
-        SvTAINTED_off(dollar_zero); 
+    
+    {
+        GV *gv = gv_fetchpv("0", TRUE, SVt_PV);
+        ENTER;
+        save_scalar(gv); /* local $0 */
+        sv_setpv_mg(GvSV(gv), directive->filename);
+        eval_pv(arg, FALSE);
+        LEAVE;
     }
-
-    ENTER;
-    save_item(dollar_zero);
-    sv_setpv(dollar_zero, directive->filename);
-    eval_pv(arg, FALSE);
-    LEAVE;
-
-    if (dollar_zero_tainted) {
-        SvTAINTED_on(dollar_zero);
-    }
-
+    
     if (SvTRUE(ERRSV)) {
-        SV *strict;
-        if ((strict = MP_STRICT_PERLSECTIONS_SV) && SvTRUE(strict)) {
-            return SvPVX(ERRSV);
+        SV *strict = MP_STRICT_PERLSECTIONS_SV;
+        if (strict && SvTRUE(strict)) {
+            char *error = SvPVX(ERRSV);
+            MP_PERL_RESTORE_CONTEXT;
+            return error;
         }
         else {
             modperl_log_warn(s, apr_psprintf(p, "Syntax error at %s:%d %s", 
                                              directive->filename, 
                                              directive->line_num, 
                                              SvPVX(ERRSV)));
-
         }
     }
     
     if (handler) {
-        SV *saveconfig;
+        int status;
+        SV *saveconfig = MP_PERLSECTIONS_SAVECONFIG_SV;
+        AV *args = Nullav;
+        
         modperl_handler_make_args(aTHX_ &args,
                                   "Apache::CmdParms", parms,
                                   "APR::Table", options,
@@ -465,7 +568,7 @@ MP_CMD_SRV_DECLARE(perldo)
 
         SvREFCNT_dec((SV*)args);
 
-        if (!(saveconfig = MP_PERLSECTIONS_SAVECONFIG_SV) || !SvTRUE(saveconfig)) {
+        if (!(saveconfig && SvTRUE(saveconfig))) {
             HV *symtab = (HV*)gv_stashpv(pkg_name, FALSE);
             if (symtab) {
                 modperl_clear_symtab(aTHX_ symtab);
@@ -473,12 +576,15 @@ MP_CMD_SRV_DECLARE(perldo)
         }
         
         if (status != OK) {
-            return SvTRUE(ERRSV) ? SvPVX(ERRSV) :
+            char *error = SvTRUE(ERRSV) ? SvPVX(ERRSV) :
                 apr_psprintf(p, "<Perl> handler %s failed with status=%d",
                              handler->name, status);
+            MP_PERL_RESTORE_CONTEXT;
+            return error;
         }
     }
 
+    MP_PERL_RESTORE_CONTEXT;
     return NULL;
 }
 
@@ -515,7 +621,7 @@ MP_CMD_SRV_DECLARE(END)
     char line[MAX_STRING_LEN];
 
     while (!ap_cfg_getline(line, sizeof(line), parms->config_file)) {
-	/* soak up rest of the file */
+        /* soak up rest of the file */
     }
 
     return NULL;
