@@ -1,9 +1,28 @@
+/* Copyright 2001-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 #ifndef MODPERL_CMD_H
 #define MODPERL_CMD_H
 
 char *modperl_cmd_push_handlers(MpAV **handlers, const char *name,
                                 apr_pool_t *p);
 
+char *modperl_cmd_push_filter_handlers(MpAV **handlers,
+                                       const char *name,
+                                       apr_pool_t *p);
+
 char *modperl_cmd_push_httpd_filter_handlers(MpAV **handlers,
                                              const char *name,
                                              apr_pool_t *p);
