@@ -54,23 +54,22 @@ sub handler {
     my $bb = APR::Brigade->new($c->pool, $c->bucket_alloc);
 
     for (;;) {
-        my $rv = $c->input_filters->get_brigade($bb,
-                                                Apache::MODE_GETLINE);
-
-        if ($rv != APR::SUCCESS or $bb->empty) {
+        my $rv = $c->input_filters->get_brigade($bb, Apache::MODE_GETLINE);
+        if ($rv != APR::SUCCESS && $rv != APR::EOF) {
             my $error = APR::strerror($rv);
-            unless ($rv == APR::EOF) {
-                warn "[echo_filter] get_brigade: $error\n";
-            }
-            $bb->destroy;
+            warn __PACKAGE__ . ": get_brigade: $error\n";
             last;
         }
 
+        last if $bb->empty;
+
         my $b = APR::Bucket::flush_create($c->bucket_alloc);
         $bb->insert_tail($b);
         $c->output_filters->pass_brigade($bb);
     }
 
+    $bb->destroy;
+
     Apache::OK;
 }
 
