@@ -25,7 +25,7 @@ sub handler {
 
     $r->content_type('text/plain');
     $r->print('ok');
-
+    $r->pnotes(items => ["cleanup"," ok"]);
     $r->push_handlers(PerlCleanupHandler => \&cleanup2);
 
     return Apache::OK;
@@ -34,8 +34,11 @@ sub handler {
 sub cleanup1 {
     my $r = shift;
 
+    my $items = $r->pnotes('items');
+    die "no items" unless $items;
+    my $item = $items ? $items->[0] : '';
     #warn "cleanup CALLED\n";
-    t_write_file(get_file(), "cleanup");
+    t_write_file(get_file(), $item);
 
     return Apache::OK;
 }
@@ -43,8 +46,10 @@ sub cleanup1 {
 sub cleanup2 {
     my $r = shift;
 
+    my $items = $r->pnotes('items');
+    my $item = $items ? $items->[1] : '';
     #warn "cleanup2 CALLED\n";
-    t_append_file(get_file(), " ok");
+    t_append_file(get_file(), $item);
 
     return Apache::OK;
 }
