@@ -12,7 +12,7 @@ use Apache::Const -compile => qw(OK :options :override :satisfy);
 sub handler {
     my $r = shift;
 
-    plan $r, tests => 8;
+    plan $r, tests => 10;
 
     $r->allow_methods(1, qw(GET POST));
 
@@ -28,7 +28,15 @@ sub handler {
 
     ok $r->auth_name eq 'modperl';
 
+    $r->auth_name('modperl_test');
+    ok $r->auth_name eq 'modperl_test';
+    $r->auth_name('modperl');
+
     ok $r->auth_type eq 'none';
+    
+    $r->auth_type('Basic');
+    ok $r->auth_type eq 'Basic';
+    $r->auth_type('none');
 
     ok !$r->some_auth_required;
 
