@@ -18,7 +18,7 @@ sub handler {
 
     my $cfg = Apache::Test::config();
 
-    plan $r, tests => 13;
+    plan $r, tests => 14;
 
     my $top_module = Apache::Module->top_module;
 
@@ -92,6 +92,9 @@ sub handler {
     ok t_cmp(0, Apache::Module::loaded('Apache__Module_foo.foo'),
              "Apache::Module::loaded('Apache__Module_foo.foo')");
 
+    ok t_cmp(0, Apache::Module::loaded(''),
+             "Apache::Module::loaded('')");
+
     Apache::OK;
 }
 
