@@ -1,3 +1,18 @@
+/* Copyright 2001-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 #define mp_xs_sv2_modperl_filter(sv) \
 ((SvROK(sv) && (SvTYPE(SvRV(sv)) == SVt_PVMG)) \
 || (Perl_croak(aTHX_ "argument is not a blessed reference"),0) ? \
@@ -155,7 +170,28 @@ static MP_INLINE SV *mpxs_Apache__Filter_ctx(pTHX_
 {
     modperl_filter_ctx_t *ctx = (modperl_filter_ctx_t *)(filter->ctx);
 
+    /* XXX: is it possible that the same filter, during a single
+     * request or connection cycle, will be invoked by different perl
+     * interpreters? if that happens we are in trouble, if we need to
+     * return an SV living in a different interpreter. may be there is
+     * a way to use one of the perl internal functions to clone an SV
+     * (and it can contain any references)
+     */
+    
     if (data != Nullsv) {
+        if (ctx->data) {
+            if (SvOK(ctx->data) && SvREFCNT(ctx->data)) {
+                /* release the previously stored SV so we don't leak
+                 * an SV */
+                SvREFCNT_dec(ctx->data);
+            }
+        }
+
+#ifdef USE_ITHREADS
+        if (!ctx->perl) {
+            ctx->perl = aTHX;
+        }
+#endif
         ctx->data = SvREFCNT_inc(data);
     }
 
